/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import java.util.List;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.minfo.BedHeightSingleData;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.exports.XYChartGenerator;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.model.BedHeightSingleValue;
import org.dive4elements.river.themes.ThemeDocument;
import org.dive4elements.river.utils.RiverUtils;

public class BedHeightProcessor extends DefaultProcessor {

    private final static Logger logger =
            Logger.getLogger(BedHeightProcessor.class);

    public static final String I18N_AXIS_LABEL_DEFAULT
        = "Sohlhoehe [m]";
    public static final String I18N_AXIS_LABEL =
        "chart.bedheight.height.yaxis.label";

    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible) {
        XYSeries series = prepareSeries(bundle, theme, generator.getCallContext());
        generator.addAxisSeries(series, axisName, visible);
    }

    @Override
    public void doOut(
            XYChartGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument theme,
            boolean visible,
            int index
    ) {
        XYSeries series = prepareSeries(bundle, theme, generator.getCallContext());
        generator.addAxisSeries(series, index, visible);
    }

    /** Prepare an series, independent of axis. */
    private XYSeries prepareSeries(ArtifactAndFacet bundle,
        ThemeDocument theme, CallContext context) {
        Object data = bundle.getData(context);
        XYSeries series = new StyledXYSeries(bundle.getFacetDescription(),
                theme);
        if (data instanceof BedHeightSingleData) {
            BedHeightSingleData bData = (BedHeightSingleData)data;
            double[] heights  = bData.getHeights();
            double[] stations = bData.getStations().toNativeArray();

            logger.debug("doBedheightSingleOut");

            for (int i = 0; i < heights.length; i++) {
                series.add(stations[i], heights[i], false);
            }
        } else if (data instanceof List<?>) {
            List<BedHeightSingleValue> bData = (List<BedHeightSingleValue>)data;

            logger.debug("doBedheightSingleValueOut");

            for(BedHeightSingleValue bvalue: bData) {
                series.add(bvalue.getStation(), bvalue.getHeight());
            }
        }
        else {
            logger.error("Unknown data type " + data.getClass());
        }
        return series;
    }


    @Override
    public boolean canHandle(String facettype) {
        return FacetTypes.BEDHEIGHT.equals(facettype);
    }

    @Override
    public String getAxisLabel(DiagramGenerator generator) {
        D4EArtifact flys = (D4EArtifact) generator.getMaster();

        String unit = RiverUtils.getRiver(flys).getWstUnit().getName();

        return generator.msg(
                I18N_AXIS_LABEL,
                I18N_AXIS_LABEL_DEFAULT,
                new Object[] { unit });
    }
}
